-- constants
local MAX_ENTRIES_SHOWN = 6;
local SHARE_BAR_WIDTH = 265;

local SPOT_RADIUS = 10;
local ENTRY_MIN_RADIUS = 5;
local ENTRY_MAX_RADIUS = 40;
local ENTRY_MOD_RADIUS = 5;

local MSG_ADDON_NAME = "TravelGuide";
local MSG_REQUEST = "REQ";
local MSG_INCOMING = "INC";
local MSG_INCOMING_SPLIT = "ISP";
local MSG_ACCEPT = "ACC";
local MSG_REJECT = "REJ";
local MSG_START = "STR";
local MSG_END = "END";
local MSG_DIVIDER = "\t";
local MSG_SEND_DELAY = 0.25

-- NOTE: icon scaling is disabled
--local ICON_CONVERSION_FACTOR = 1.0;

local KEYWORD_LOCKED = "Locked";
local KEYWORD_UNLOCKED = "Unlocked";

local TYPE_NORMAL = 0;
local TYPE_EXPAND = 1;
local TYPE_COLLAPSE = 2;

local TAB_GUIDE = 0;
local TAB_ENTRY = 1;

local CONN_SEND = 1;
local CONN_RECIEVE = 2;

local NO_GUIDE_TEXT = "No Guide Selected";
local CREATE_ENTRY_NO_GUIDE_ERROR = "Cannot Create Entry: You have not selected a guide or the guide you are currently viewing is locked.";
local CREATE_ENTRY_NO_LOCATION_ERROR = "Cannot Create Entry: Map coordinates are not available for your current location.";

local TRAVEL_GUIDE_ENTRY_HEIGHT = 16;
local ENTRY_BRIEF_LINE_LIMIT = 3;
local POPUP_SHOW_TIME = 3;

local VIEW_LOCATION_FORMAT = "Located in %s";
local BASE_LOCATION_FORMAT_ZONE = "%s (%.2f, %.2f), %i ft. radius";
local BASE_LOCATION_FORMAT_SUBZONE = "%s, %s (%.2f, %.2f), %i ft. radius";
local DISCOVER_POPUP_FORMAT = "%s Discovered";
local VISIT_POPUP_FORMAT = "Visiting %s";
local SEND_PROGRESS_FORMAT = "Sending...%i%%";
local RECIEVE_PROGRESS_FORMAT = "Recieving...%i%%";

local MINIMAP_ICON_NAME_FORMAT = "tgMinimapIcon_%i";
local WORLD_MAP_ICON_NAME_FORMAT = "tgWorldMapIcon_%i";

local MSG_RECIPIENT_LABEL = "Recipient";
local MSG_SENDER_LABEL = "Sender";

local NO_RECIPIENT_LABEL = "Recipient Required";
local REQUEST_PERMISSION_LABEL = "Requesting";
local REQUEST_ACCEPTED_LABEL = "Request Accepted";
local REQUEST_REJECTED_LABEL = "Request Denied";
local SEND_COMPLETE_LABEL = "Send Complete";
local RECIEVE_COMPLETE_LABEL = "Recieve Complete";
local CONNECTION_FAILED_LABEL = "Connection Failed";
local SHARE_DONE_BUTTON_LABEL = "Done";
local SHARE_CANCEL_BUTTON_LABEL = "Cancel";

StaticPopupDialogs["TRAVEL_GUIDE_RESET_GUIDE_CONFIRM"] = {
	text = "Do you want to reset all progress that was made for this guide?",
	button1 = "Yes",
	button2 = "No",
	OnAccept = function()
	  TravelGuide:guiResetGuideConfirm();
	end,
	timeout = 0,
	whileDead = 1,
	hideOnEscape = 1
};

local TRAVEL_GUIDE_RECIEVE_GUIDE_DIALOG_FORMAT = "%s is requesting to send you a TravelGuide named %q. Do you accept?"
StaticPopupDialogs["TRAVEL_GUIDE_RECIEVE_GUIDE_CONFIRM"] = {
	text = "",
	button1 = "Accept",
	button2 = "Reject",
	OnAccept = function()
	  TravelGuide:guiAcceptGuideShare();
	end,
	OnCancel = function()
	  TravelGuide:guiRejectGuideShare();
	end,
	timeout = 0,
	whileDead = 1,
	hideOnEscape = 0
};

-- saved globals
tgSavedCurrentGuideIndex = nil;
tgSavedCurrentEntryIndex = nil;
tgSavedGuides = {};
tgSavedPrefs = {
	edgePosition = -100;
};

-- start with two expandable entries
tgSavedGuideTable =
{
	[1] = {name = KEYWORD_LOCKED, type = TYPE_EXPAND, index = -1},
	[2] = {name = KEYWORD_UNLOCKED, type = TYPE_EXPAND, index = -1}
}

tgSavedEntryTable = {};
tgSavedViewTable = {};

-- local variables
local currentTab = nil;
local currentMode = nil;
local currentView = nil;
local currentEntry = nil;
local currentGuide = nil;
local currentIcons = {};

-- recycled icons
local iconCount = 0;
local recycleIcons = {};

-- sharing
local connectTarget = nil;
local connectState = nil;
local connectInputLock = false;
local sendQueue = nil;
local recieveQueue = nil;
local recievePartial = nil;

local sendProgress = nil;
local sendInterval = nil;
local recieveProgress = nil;
local recieveGoal = nil;

-- tooltip used for world map
local worldMapTooltip;

local entryScroll = 0;
local guideScroll = 0;

-- position tracking
local currC, currZ, currX, currY, currZone, currSubZone;
local zoneChanged;

local updateFadeIn = false;
local updateFadeOut = false;
local updateAlpha = 0;

local popUpFadeOut = false;
local popUpTimer = 0;

local edgeDragging = false;

--[[
	guide main class
]]
local Astrolabe = DongleStub("Astrolabe-0.4")
TravelGuide = {};

--[[
	guide related functions
]]

function TravelGuide:modifyGuide(index, name, details, locked, password)

	local guide;
	local dropDownIndex;
	
	-- insert guide
	if (index == nil) then
		guide = {["name"] = "Untitled Guide", ["details"] = "", ["locked"] = false, ["password"] = "", ["entries"] = {}};
		table.insert(tgSavedGuides, guide);
		index = table.getn(tgSavedGuides);
	else
		guide = tgSavedGuides[index];
	end
	
	if(name ~= nil) then
		guide.name = strtrim(name, " ");
	end
	
	if(details ~= nil) then
		guide.details = strtrim(details, " ");
	end
	
	if(locked ~= nil) then
		guide.locked = locked;
	end
	
	if(password ~= nil) then
		guide.password = strtrim(password, " ");
	end
	
	return index;
end

function TravelGuide:currentGuideLocked()
	return (currentGuide == nil or currentGuide.locked);
end

function TravelGuide:deleteGuide(index)

	table.remove(tgSavedGuides, index);
	
	--[[
	if (tgSavedCurrentGuideIndex > table.getn(tgSavedGuides)) then
		tgSavedCurrentGuideIndex = tgSavedCurrentGuideIndex - 1;
		currentGuide = tgSavedGuides[tgSavedCurrentGuideIndex];
	end
	
	if (tgSavedCurrentGuideIndex <= 0) then
		tgSavedCurrentGuideIndex = nil;
		currentGuide = nil;
	end
	]]
	
	tgSavedCurrentGuideIndex = nil;
	currentGuide = nil;
end
	
function TravelGuide:swapGuide(index, direction)
	
	local target = index + direction;
	
	while (target > 0 and target <= table.getn(tgSavedGuides) and tgSavedGuides[target].locked ~= tgSavedGuides[target].locked) do
		target = target + direction;
	end
	
	-- can't swap top or bottom
	if (target < 1 or target > table.getn(tgSavedGuides)) then
		return;
	end
	
	local swap = tgSavedGuides[index];
	tgSavedGuides[index] = tgSavedGuides[target];
	tgSavedGuides[target] = swap;	
	
	if (index == tgSavedCurrentGuideIndex) then
		tgSavedCurrentGuideIndex = target;
	elseif (target == tgSavedCurrentGuideIndex) then
		tgSavedCurrentGuideIndex = index;
	end
end

function TravelGuide:setGuide(index)

	local rebuildEntryTable = (index ~= tgSavedCurrentGuideIndex);
	
	if (index ~= nil) then
		tgSavedCurrentGuideIndex = index;
	else
		index = tgSavedCurrentGuideIndex;
	end
	
	currentGuide = tgSavedGuides[index];
	
	if (rebuildEntryTable) then
		self:guiCreateEntryTable();
		self:guiTrackClear();
	end
	
	if (tgViewer:IsShown()) then
		self:guiShowViewOverview();
		self:guiShowViewData();
	elseif (tgEditor:IsShown()) then
		self:guiShowGuideOverview();
		self:guiShowGuideData();
	end
end

--[[
	entry related functions
]]
function TravelGuide:modifyEntry(index, name, brief, details, zone, subzone, c, z, x, y, r)

	local entry;
	
	-- insert guide
	if (index == nil) then
		entry = {["name"] = "Untitled Entry", ["brief"] = "", ["details"] = "", ["zone"] = "", ["subzone"] = "", ["c"] = 0, ["z"] = 0, ["x"] = 0, ["y"] = 0, ["r"] = SPOT_RADIUS, ["unlocked"] = false};
		table.insert(currentGuide.entries, entry);
		index = table.getn(currentGuide.entries);
	else
		entry = currentGuide.entries[index];
	end
	
	if (name ~= nil) then
		entry.name = strtrim(name, " ");
	end
	
	if (brief ~= nil) then
		entry.brief = strtrim(brief, " ");
	end
	
	if (details ~= nil) then
		entry.details = strtrim(details, " ");
	end
	
	if (zone ~= nil) then
		entry.zone = zone;
	end
	
	if (subzone ~= nil) then
		entry.subzone = subzone;
	end
	
	if(c ~= nil) then
		entry.c = c;
		entry.z = z;
		entry.x = x;
		entry.y = y;
	end
	
	if (r ~= nil) then
		entry.r = r;
	end
	
	return index;
end

function TravelGuide:getPositionText(entry)

	if (entry == nil or entry.c == 0) then
		return "unknown";
	else
		if (entry.subzone == "") then
			return string.format(BASE_LOCATION_FORMAT_ZONE, entry.zone, entry.x * 100, entry.y * 100, entry.r * 2);
		else
			return string.format(BASE_LOCATION_FORMAT_SUBZONE, entry.subzone, entry.zone, entry.x * 100, entry.y * 100, entry.r * 2);
		end
	end
end

function TravelGuide:getEditPositionText(entry)

	if (entry == nil or entry.c == 0) then
		return "unknown";
	else
		return string.format(BASE_LOCATION_FORMAT_ZONE, entry.zone, entry.x * 100, entry.y * 100, entry.r * 2);
	end
end

function TravelGuide:deleteEntry(index)
	
	-- remove icon
	self:guiRemoveIcon(currentGuide.entries[index]);
	
	-- remove view if it is being deleted
	if (currentView == currentGuide.entries[index]) then
		self:setView(nil);
	end
	
	table.remove(currentGuide.entries, index);
	
	--[[
	if (tgSavedCurrentEntryIndex > table.getn(currentGuide.entries)) then
		tgSavedCurrentEntryIndex = tgSavedCurrentEntryIndex - 1;
		currentEntry = currentGuide.entries[tgSavedCurrentEntryIndex];
	end
	
	if (tgSavedCurrentEntryIndex <= 0) then	
		tgSavedCurrentEntryIndex = nil;
		currentEntry = nil;
	end
	]]
	
	tgSavedCurrentEntryIndex = nil;
	currentEntry = nil;
end
	
function TravelGuide:swapEntry(index, direction)
	
	local target = index + direction;
	
	while (target > 0 and target <= table.getn(currentGuide.entries) and currentGuide.entries[target].zone ~= currentGuide.entries[target].zone) do
		target = target + direction;
	end
	
	-- can't swap top or bottom
	if (target < 1 or target > table.getn(currentGuide.entries)) then
		return;
	end	
	
	local swap = currentGuide.entries[index];
	currentGuide.entries[index] = currentGuide.entries[target];
	currentGuide.entries[target] = swap;
		
	if (index == tgSavedCurrentEntryIndex) then
		tgSavedCurrentEntryIndex = target;
	elseif (target == tgSavedCurrentEntryIndex) then
		tgSavedCurrentEntryIndex = index;
	end
end

function TravelGuide:setEntry(index)
	tgSavedCurrentEntryIndex = index;
	
	if (currentGuide ~= nil) then
	
		-- clean up previous entry
		if (currentEntry ~= nil) then
			local oldIcon = self:guiFindIcon(currentEntry);
			if oldIcon then
				if (oldIcon.temp) then
					self:guiRemoveIcon(currentEntry);
				else				
					getglobal(oldIcon.minimap:GetName().."Normal"):Show();
					getglobal(oldIcon.minimap:GetName().."Selected"):Hide();
					getglobal(oldIcon.worldMap:GetName().."Normal"):Show();
					getglobal(oldIcon.worldMap:GetName().."Selected"):Hide();
				end
			end
		end
		
		currentEntry = currentGuide.entries[tgSavedCurrentEntryIndex];
		
		local newIcon = self:guiFindIcon(currentEntry);
		if newIcon then
			getglobal(newIcon.minimap:GetName().."Normal"):Hide();
			getglobal(newIcon.minimap:GetName().."Selected"):Show();
			getglobal(newIcon.worldMap:GetName().."Normal"):Hide();
			getglobal(newIcon.worldMap:GetName().."Selected"):Show();
		else
			self:guiAddIcon(currentEntry, true);
		end
	end
	
	if (tgViewer:IsShown()) then
		self:guiShowViewOverview();
		self:guiShowViewData();
	elseif (tgEditor:IsShown()) then
		self:guiShowEntryOverview();
		self:guiShowEntryData();
	end		
end

--[[
	view related functions
]]

function TravelGuide:endView()

	if (currentView ~= nil) then
		currentView = nil;
		updateFadeOut = true;
		updateFadeIn = false;
	end	
end

function TravelGuide:startView(view)

	if (currentView ~= view) then
		currentView = view;
		updateFadeIn = true;
		updateFadeOut = false;
		
		if (currentView.unlocked) then
			self:guiShowPopUp(string.format(VISIT_POPUP_FORMAT, view.name));
		else
			self:guiShowPopUp(string.format(DISCOVER_POPUP_FORMAT, view.name));
			PlaySound("TaxiNodeDiscovered");
			view.unlocked = true;
			
			-- update display
			self:guiShowViewOverview();			
		end
	end
	
	-- keep view updated
	self:guiSetLiveEntry(view);
end

function TravelGuide:setView(view)

	if (view == nil) then
		self:endView();
	else
		self:startView(view);
	end
end

--[[
	GUI handlers
]]

function TravelGuide:guiInit()

	-- initialize tooltip names
	tgContainerTab1.tooltip = "TravelGuide Viewer";
	tgContainerTab2.tooltip = "TravelGuide Editor";
	
	-- initialize additional parameters
	tgContainerTab1.target = "tgViewer";
	tgContainerTab2.target = "tgEditor";
	
	-- hide all frames
	TravelGuide:guiSetGuideTab();
	
	tgContainer:Hide();
	tgViewer:Hide();
	tgEditor:Hide();
	
	-- register events
	this:RegisterEvent("WORLD_MAP_UPDATE");
	this:RegisterEvent("ZONE_CHANGED_NEW_AREA");
	this:RegisterEvent("MINIMAP_ZONE_CHANGED");
	this:RegisterEvent("ADDON_LOADED");
	this:RegisterEvent("CHAT_MSG_ADDON");
	
	zoneChanged = true;
	currentMode = tgContainerTab1;
	
	worldMapTooltip = CreateFrame("GameTooltip", "tgWorldMapTooltip", UIParent, "GameTooltipTemplate");
	
	-- add slash commands
	 SlashCmdList["TRAVELGUIDE"] = TravelGuide.guiSlashCommand;
	 SLASH_TRAVELGUIDE1 = "/tg";
	 SLASH_TRAVELGUIDE2 = "/travelguide";
end

function TravelGuide:guiSlashCommand(command)
	-- show guide
	tgContainer:Show();	
	TravelGuide:guiSetFrame(currentMode);
end

function TravelGuide:guiEvent()

	if (event == "WORLD_MAP_UPDATE" or event == "ZONE_CHANGED_NEW_AREA" or event == "MINIMAP_ZONE_CHANGED") then
		-- update zone
		zoneChanged = true;
	elseif (event == "ADDON_LOADED") then	
		-- retrieve current guide/current entry
		if (tgSavedCurrentGuideIndex) then
			self:setGuide(tgSavedCurrentGuideIndex);
		end
		if (tgSavedCurrentEntryIndex) then
			self:setEntry(tgSavedCurrentEntryIndex);
		end
		
		-- move minimap icon
		self:moveEdgeIcon();
	elseif (event == "CHAT_MSG_ADDON" and arg1 == MSG_ADDON_NAME) then
		-- interpret addon message
		self:interpretMessage(arg2, arg4);
	end
end

function TravelGuide:guiUpdate(interval)

	if (zoneChanged) then
		-- update zone name
		SetMapToCurrentZone();
		
		currZone = GetZoneText();
		currSubZone = GetSubZoneText();
	end
	
	-- get player position
	currC, currZ, currX, currY = Astrolabe:GetCurrentPlayerPosition();
	
	-- check player location against spots in current guide
	local i;
	local view = nil;
	local dist = nil;
	
	if (currentView ~= nil) then
		-- try and maintain spot
		dist = Astrolabe:ComputeDistance(currC, currZ, currX, currY, currentView.c, currentView.z, currentView.x, currentView.y)
		if (dist ~= nil and dist < currentView.r) then
			view = currentView;
		end
	end
	
	if (view == nil and currentGuide ~= nil) then
		-- find new spot
		for i = 1, table.getn(currentGuide.entries) do
			local entry = currentGuide.entries[i];
			dist = Astrolabe:ComputeDistance(currC, currZ, currX, currY, entry.c, entry.z, entry.x, entry.y)
			if (dist ~= nil and dist < entry.r) then
				view = entry;
			end
		end		
	end
	
	self:setView(view);		
	self:guiUpdateLiveAlpha(interval);
	
	-- entry create icon visibility
	if (currentGuide ~= nil and (not currentGuide.locked)) then
		tgCompanion:Show();
	else
		tgCompanion:Hide();
	end
	
	-- sharing
	if (sendProgress) then
		sendInterval = sendInterval - interval;
		if (sendInterval < 0) then
			sendInterval = MSG_SEND_DELAY;
			
			-- send next chunk
			self:sendMessage(sendQueue[sendProgress]);
			
			-- update display
			local percentage = sendProgress/table.getn(sendQueue);
			tgShareProgressBar:SetWidth(percentage * SHARE_BAR_WIDTH);
			tgShareProgress:SetText(string.format(SEND_PROGRESS_FORMAT, percentage * 100));
			
			sendProgress = sendProgress + 1;
			
			-- ending send
			if (sendProgress > table.getn(sendQueue)) then
				tgShareProgress:SetText(SEND_COMPLETE_LABEL);
				tgShareCancel:SetText(SHARE_DONE_BUTTON_LABEL);
				tgShareStart:Enable();
				self:connectInputUnlock();
				sendProgress = nil;
			end
		end
	end
end

function TravelGuide:guiSetFrame(tab)
	
	-- track current mode
	currentMode = tab;
	
	-- close all frames, unblink all check marks
	tgViewer:Hide();
	tgEditor:Hide();
	tgContainerTab1:SetChecked(false);
	tgContainerTab2:SetChecked(false);
	
	-- set the frame designated by the target
	getglobal(tab.target):Show();
	tab:SetChecked(true);
	
	-- set the guide/entries
	if (currentMode == tgContainerTab2) then
		if (currentTab == TAB_GUIDE) then
			self:guiUpdateGuideOverview();
			self:guiShowGuideOverview();
			self:guiShowGuideData();
		elseif (currentTab == TAB_ENTRY) then
			self:guiUpdateEntryOverview();
			self:guiShowEntryOverview();
			self:guiShowEntryData();
		end
	end
end

--[[
	viewer
]]

function TravelGuide:guiSetViewerOverview(btn)
	self:guiSetViewOverview(btn:GetID());
	self:guiShowViewOverview();
end

function TravelGuide:guiResetViewerOverviewColor(btn)
end

--[[
	editor general
]]

function TravelGuide:guiSetEditorOverview(btn)	

	if (currentTab == TAB_GUIDE) then
		self:guiSetGuideOverview(btn:GetID());
		self:guiShowGuideOverview();
	elseif (currentTab == TAB_ENTRY) then
		self:guiSetEntryOverview(btn:GetID());
		self:guiShowEntryOverview();
	end
end

function TravelGuide:guiResetEditorOverviewColor(btn)
end

function TravelGuide:guiEditShift(direction)

	if (currentTab == TAB_GUIDE) then
		self:swapGuide(tgSavedCurrentGuideIndex, direction);
		self:guiUpdateGuideOverview();
		self:guiShowGuideOverview();
	elseif (currentTab == TAB_ENTRY) then
		self:swapEntry(tgSavedCurrentEntryIndex, direction);
		self:guiUpdateEntryOverview();
		self:guiShowEntryOverview();
	end
end

function TravelGuide:guiDelete()
	
	if (currentTab == TAB_GUIDE) then
		self:guiDeleteGuide();
	elseif (currentTab == TAB_ENTRY) then
		self:guiDeleteEntry();
	end
end

function TravelGuide:guiSave()
	
	if (currentTab == TAB_GUIDE) then
		self:guiSaveGuide();
	elseif (currentTab == TAB_ENTRY) then
		self:guiSaveEntry();
	end
end

function TravelGuide:guiSetGuideTab()
	-- TODO : Save Entry Scroll Position
	
	PanelTemplates_SetTab(tgEditor, 1);	
	currentTab = TAB_GUIDE;
	
	tgEditorOverviewShiftUp:Hide();
	tgEditorOverviewShiftDown:Hide();
	tgEditorOverviewZoomIn:Hide();
	tgEditorOverviewZoomOut:Hide();	
	tgEditorEntryResetPosition:Hide();
	
	tgEditorEntryFrame:Hide();
	tgEditorGuideFrame:Show();
	tgEditorGuideCreate:Show();
	tgEditorEntryCreate:Hide();
end

function TravelGuide:guiSetEntryTab()
	-- TODO : Save Guide Scroll Position
	
	PanelTemplates_SetTab(tgEditor, 2);
	currentTab = TAB_ENTRY;
	
	tgEditorOverviewShiftUp:Hide();
	tgEditorOverviewShiftDown:Hide();
	tgEditorOverviewZoomIn:Hide();
	tgEditorOverviewZoomOut:Hide();	
	tgEditorEntryResetPosition:Hide();
	
	tgEditorGuideFrame:Hide();
	tgEditorGuideCreate:Hide();
	tgEditorEntryFrame:Show();
	tgEditorEntryCreate:Show();
end

function TravelGuide:guiEditScroll(self, offset)
	
	if(currentTab == TAB_GUIDE) then
		FauxScrollFrame_OnVerticalScroll(self, offset, TRAVEL_GUIDE_ENTRY_HEIGHT, TravelGuide.guiShowGuideOverview);
	else
		FauxScrollFrame_OnVerticalScroll(self, offset, TRAVEL_GUIDE_ENTRY_HEIGHT, TravelGuide.guiShowEntryOverview);
	end
end

--[[
	guide frame
]]

function TravelGuide:guiSetGuideOverview(id)

	local index = id + FauxScrollFrame_GetOffset(tgEditorOverviewScrollFrame);
	local i;
	
	-- expand
	if (tgSavedGuideTable[index].type == TYPE_EXPAND) then	
		-- change current icon into a collapse
		tgSavedGuideTable[index].type = TYPE_COLLAPSE;
		
		-- expand content
		local pushed = 0;
		
		if (tgSavedGuideTable[index].name == KEYWORD_LOCKED) then
			for i = 1, table.getn(tgSavedGuides) do
				if (tgSavedGuides[i].locked) then
					table.insert(tgSavedGuideTable, index+pushed+1, {name = tgSavedGuides[i].name, type = TYPE_NORMAL, index = i});
					pushed = pushed + 1;
				end
			end
		elseif (tgSavedGuideTable[index].name == KEYWORD_UNLOCKED) then
			for i = 1, table.getn(tgSavedGuides) do
				if not (tgSavedGuides[i].locked) then
					table.insert(tgSavedGuideTable, index+pushed+1, {name = tgSavedGuides[i].name, type = TYPE_NORMAL, index = i});
					pushed = pushed + 1;
				end
			end
		end
		
		-- expand failed
		if (pushed == 0) then
			tgSavedGuideTable[index].type = TYPE_EXPAND;
		end
		
	elseif (tgSavedGuideTable[index].type == TYPE_COLLAPSE) then
		-- change current icon to an expand
		tgSavedGuideTable[index].type = TYPE_EXPAND;
		
		-- collapse content
		while (index+1 <= table.getn(tgSavedGuideTable) and tgSavedGuideTable[index+1].type == TYPE_NORMAL) do
			table.remove(tgSavedGuideTable, index+1);
		end
	elseif (tgSavedGuideTable[index].type == TYPE_NORMAL) then
		-- changed current guide to index
		self:setGuide(tgSavedGuideTable[index].index);
	end
	
	-- update display
	self:guiShowGuideOverview();
end

function TravelGuide:guiUpdateGuideOverview()

	-- duUtil:message("Table Size "..table.getn(tgSavedGuideTable));

	local keyword = nil;
	if (currentGuide ~= nil) then
		if (currentGuide.locked) then
			keyword = KEYWORD_LOCKED;
		else
			keyword = KEYWORD_UNLOCKED;
		end
	end
	
	local i = 1;
	while i <= table.getn(tgSavedGuideTable) do
		-- duUtil:message("Handling Index "..i);
		if (tgSavedGuideTable[i].type == TYPE_EXPAND and tgSavedGuideTable[i].name == keyword) then
			-- toggle entry once to expand it
			self:guiSetGuideOverview(i - FauxScrollFrame_GetOffset(tgEditorOverviewScrollFrame));
		elseif (tgSavedGuideTable[i].type == TYPE_COLLAPSE) then
			-- toggle entry twice to refresh it
			self:guiSetGuideOverview(i - FauxScrollFrame_GetOffset(tgEditorOverviewScrollFrame));
			self:guiSetGuideOverview(i - FauxScrollFrame_GetOffset(tgEditorOverviewScrollFrame));
		end
		
		i = i + 1;
	end
end
	
function TravelGuide:guiShowGuideOverview()

	-- update scroll frame
	FauxScrollFrame_Update(tgEditorOverviewScrollFrame, table.getn(tgSavedGuideTable), MAX_ENTRIES_SHOWN, TRAVEL_GUIDE_ENTRY_HEIGHT, nil, nil, nil, tgEditorOverviewHighlightFrame, 293, 316);
	
	-- update overviews
	tgEditorOverviewHighlightFrame:Hide();
	
	local i;
	for i = 1, MAX_ENTRIES_SHOWN do
		local index = i + FauxScrollFrame_GetOffset(tgEditorOverviewScrollFrame);
		
		local title = getglobal("tgEditorOverview"..i);
		local checkMark = getglobal("tgEditorOverview"..i.."Check");
		local normalText = getglobal("tgEditorOverview"..i.."NormalText");
		local highlight = getglobal("tgEditorOverview"..i.."Highlight");
		
		if (index <= table.getn(tgSavedGuideTable)) then
		
			local data = tgSavedGuideTable[index];
			local color;
			
			-- icons
			if (data.type == TYPE_EXPAND) then
				title:SetText(data.name);
				title:SetNormalTexture("Interface\\Buttons\\UI-PlusButton-Up");
				highlight:SetTexture("Interface\\Buttons\\UI-PlusButton-Hilight");
				color = QuestDifficultyColor["header"]
			elseif (data.type == TYPE_COLLAPSE) then
				title:SetText(data.name);
				title:SetNormalTexture("Interface\\Buttons\\UI-MinusButton-Up");
				highlight:SetTexture("Interface\\Buttons\\UI-PlusButton-Hilight");
				color = QuestDifficultyColor["header"]
			else
				title:SetText("  "..data.name);
				title:SetNormalTexture("");
				highlight:SetTexture("");
				color = QuestDifficultyColor["difficult"]
			end
			
			title:SetNormalFontObject(color.font);
			title.r = color.r;
			title.g = color.g;
			title.b = color.b;
			
			if (data.index == tgSavedCurrentGuideIndex) then
				tgEditorOverviewHighlightFrame:SetPoint("TOPLEFT", "tgEditorOverview"..i, "TOPLEFT", 0, 0);
				tgEditorOverviewHighlight:SetVertexColor(title.r, title.g, title.b);
				tgEditorOverviewHighlightFrame:Show();
				title:LockHighlight();
			else
				title:UnlockHighlight();
			end
			
			title:Show();
		else
			title:Hide();
		end
	end
	
	-- disable edit window and entry tag if no guide is selected
	if (tgSavedCurrentGuideIndex == nil) then
		tgEditorGuideFrame:Hide();
		tgEditorTab2:Hide();
	else
		tgEditorGuideFrame:Show();
		
		-- disallow entry viewing for locked guides
		if (currentGuide.locked) then
			tgEditorTab2:Hide();
		else
			tgEditorTab2:Show();
		end
	end
end

function TravelGuide:guiShowGuideData()

	tgEditorGuideTitle:ClearFocus();
	tgEditorGuideDetails:ClearFocus();

	if (tgSavedCurrentGuideIndex ~= nil) then
		tgEditorGuideTitle:SetText(currentGuide.name);
		tgEditorGuideDetails:SetText(currentGuide.details);
		tgEditorGuidePassword:SetText("");
		tgEditorGuidePasswordResultLabel:SetText("");
		
		if (currentGuide.locked) then		
			tgEditorGuideLock:SetText("Unlock");
            GameTooltip:SetText("Allow Guide Editing");
		else
			tgEditorGuideLock:SetText("Lock");
            GameTooltip:SetText("Prevent Guide Editing");
		end
		
		if (tgEditorDelete ~= nil) then
			tgEditorDelete:Enable();
			
			if (currentGuide.locked) then
				tgEditorSave:Disable();
			else
				tgEditorSave:Enable();
			end
			
			tgEditorOverviewShiftUp:Show();
			tgEditorOverviewShiftDown:Show();
			tgEditorOverviewZoomIn:Hide();
			tgEditorOverviewZoomOut:Hide();
			tgEditorEntryResetPosition:Hide();
		end
	else
		if (tgEditorDelete ~= nil) then
			tgEditorDelete:Disable();
			tgEditorSave:Disable();
			
			tgEditorOverviewShiftUp:Hide();
			tgEditorOverviewShiftDown:Hide();
			tgEditorOverviewZoomIn:Hide();
			tgEditorOverviewZoomOut:Hide();
			tgEditorEntryResetPosition:Hide();
		end
	end
end

function TravelGuide:guiCreateNewGuide()

	-- create new guide
	local index = self:modifyGuide();
	-- set to new guide
	self:setGuide(index);
	self:guiUpdateGuideOverview();
	self:guiShowGuideOverview();
end

function TravelGuide:guiClearGuideFrameFocus()
	tgEditorGuideTitle:ClearFocus();
	tgEditorGuideDetails:ClearFocus();
	tgEditorGuidePassword:ClearFocus();
end

function TravelGuide:guiLockUnlockGuide()

	TravelGuide:guiClearGuideFrameFocus();
	
	-- if locked, attempt to unlock; otherwise, lock
	if (currentGuide.locked) then
		if (currentGuide.password == tgEditorGuidePassword:GetText()) then
			self:modifyGuide(tgSavedCurrentGuideIndex, nil, nil, false, nil);			
			self:guiShowGuideData();
			tgEditorGuidePasswordResultLabel:SetText("Guide Unlocked");
		else
			self:guiShowGuideData();
			tgEditorGuidePasswordResultLabel:SetText("Incorrect Password");
		end
	elseif (not currentGuide.locked) then
		self:modifyGuide(tgSavedCurrentGuideIndex, nil, nil, true, tgEditorGuidePassword:GetText());
		self:guiShowGuideData();
		tgEditorGuidePasswordResultLabel:SetText("Guide Locked");
	end
	
	tgEditorGuidePassword:SetText("");
	self:guiUpdateGuideOverview();
	self:guiShowGuideOverview();
end

function TravelGuide:guiShareGuide()
	connectState = CONN_SEND;
	tgContainer:Hide();
	tgViewer:Hide();
	tgEditor:Hide();
	tgShare:Show();
end

function TravelGuide:guiResetGuide()
	StaticPopup_Show("TRAVEL_GUIDE_RESET_GUIDE_CONFIRM");
end

function TravelGuide:guiResetGuideConfirm()
	
	-- run through all entries in the guide
	local i;
	for i = 1, table.getn(currentGuide.entries) do
		currentGuide.entries[i].unlocked = false;
	end
end

function TravelGuide:guiDeleteGuide()

	TravelGuide:guiClearGuideFrameFocus();
	
	-- clear map icons
	self:guiTrackClear();	
	
	-- clear entry and view tables
	tgSavedEntryTable = {};
	tgSavedViewTable = {};
	
	self:guiShowViewOverview();
	self:guiShowViewData();
	
	-- delete guide
	self:deleteGuide(tgSavedCurrentGuideIndex);
	self:guiUpdateGuideOverview();
	self:guiShowGuideOverview();
	self:guiShowGuideData();
	
	-- clear current view
	self:setView(nil);
	
	-- clear current entry
	self:guiUpdateEntryOverview();
	tgSavedCurrentEntryIndex = nil;
	currentEntry = nil;
	
end

function TravelGuide:guiSaveGuide()

	TravelGuide:guiClearGuideFrameFocus();
	
	-- save guide
	self:modifyGuide(tgSavedCurrentGuideIndex, tgEditorGuideTitle:GetText(), tgEditorGuideDetails:GetText());
	self:guiUpdateGuideOverview();
	self:guiShowGuideOverview();
end

--[[
	entry frame
]]

function TravelGuide:guiSetEntryOverview(id)

	local index = id + FauxScrollFrame_GetOffset(tgEditorOverviewScrollFrame);
	local i;
	
	-- expand
	if (tgSavedEntryTable[index].type == TYPE_EXPAND) then	
		-- change current icon into a collapse
		tgSavedEntryTable[index].type = TYPE_COLLAPSE;
		
		-- expand content
		local pushed = 0;
		
		for i = 1, table.getn(currentGuide.entries) do
			if (currentGuide.entries[i].zone == tgSavedEntryTable[index].name) then
				table.insert(tgSavedEntryTable, index+pushed+1, {name = currentGuide.entries[i].name, type = TYPE_NORMAL, index = i});
				pushed = pushed + 1;
			end
		end
		
		-- expand failed
		if (pushed == 0) then
			tgSavedEntryTable[index].type = TYPE_EXPAND;
		end
		
	elseif (tgSavedEntryTable[index].type == TYPE_COLLAPSE) then
		-- change current icon to an expand
		tgSavedEntryTable[index].type = TYPE_EXPAND;
		
		-- collapse content
		while (index+1 <= table.getn(tgSavedEntryTable) and tgSavedEntryTable[index+1].type == TYPE_NORMAL) do
			table.remove(tgSavedEntryTable, index+1);
		end
	elseif (tgSavedEntryTable[index].type == TYPE_NORMAL) then
		-- changed current guide to index
		self:setEntry(tgSavedEntryTable[index].index);
	end
	
	-- update display
	--self:guiShowEntryOverview();
end

function TravelGuide:guiCreateEntryTable()

	tgSavedEntryTable = {};
	tgSavedViewTable = {};
	
	local i, j;
	for i = 1, table.getn(currentGuide.entries) do
	
		-- assume we add the category for this entry
		local newCategory = true;
		for j = 1, table.getn(tgSavedEntryTable) do
			if (currentGuide.entries[i].zone == tgSavedEntryTable[j].name) then
				newCategory = false;
			elseif (newCategory and currentGuide.entries[i].zone < tgSavedEntryTable[j].name) then
				local value = {name = currentGuide.entries[i].zone, type = TYPE_EXPAND, index = -1};
				table.insert(tgSavedEntryTable, j, value);
				table.insert(tgSavedViewTable, j, value);
				newCategory = false;
			end
		end
		
		-- add if new entry goes to the end
		if (newCategory) then
			local value = {name = currentGuide.entries[i].zone, type = TYPE_EXPAND, index = -1};
			table.insert(tgSavedEntryTable, value);
			table.insert(tgSavedViewTable, value);
		end
	end
	
	-- wipe current selection
	currentEntry = nil;
	tgSavedCurrentEntryIndex = nil;
end

function TravelGuide:guiInsertEntryCategory(category)

	local i;
	local exist = false;
	
	for i = 1, table.getn(tgSavedEntryTable) do
		
		-- duUtil:message("Handling Index "..i);
		if (tgSavedEntryTable[i].type ~= TYPE_NORMAL and tgSavedEntryTable[i].name == category) then
			-- category already exists
			exist = true;
		elseif ((not exist) and tgSavedEntryTable[i].type ~= TYPE_NORMAL and tgSavedEntryTable[i].name > category) then
			-- we are now one step below where the category should be - insert the category
			local value = {name = category, type = TYPE_EXPAND, index = -1};
			table.insert(tgSavedEntryTable, i, value);
			exist = true;
		end
	end
	
	-- insert at end
	if (not exist) then
		local value = {name = category, type = TYPE_EXPAND, index = -1};
		table.insert(tgSavedEntryTable, value);
	end
	
	exist = false;
	for i = 1, table.getn(tgSavedViewTable) do
		
		-- duUtil:message("Handling Index "..i);
		if (tgSavedViewTable[i].type ~= TYPE_NORMAL and tgSavedViewTable[i].name == category) then
			-- category already exists
			exist = true;
		elseif ((not exist) and tgSavedViewTable[i].type ~= TYPE_NORMAL and tgSavedViewTable[i].name > category) then
			-- we are now one step below where the category should be - insert the category
			local value = {name = category, type = TYPE_EXPAND, index = -1};
			table.insert(tgSavedViewTable, i, value);
			exist = true;
		end
	end
	
	-- insert at end
	if (not exist) then	
		local value = {name = category, type = TYPE_EXPAND, index = -1};
		table.insert(tgSavedViewTable, value);
	end
end

function TravelGuide:guiDeleteEntryCategory(category)

	local i;
	for i = 1, table.getn(currentGuide.entries) do
		-- entries still exist for this category
		if (currentGuide.entries[i].zone == category) then
			return;
		end
	end
	
	-- deleting category from entry table
	local i = 1;
	while i <= table.getn(tgSavedEntryTable) do
		if (tgSavedEntryTable[i].type ~= TYPE_NORMAL and tgSavedEntryTable[i].name == category) then
			-- collapse category first
			if (tgSavedEntryTable[i].type == TYPE_COLLAPSE) then
				self:guiSetEntryOverview(i - FauxScrollFrame_GetOffset(tgEditorOverviewScrollFrame));
			end
			table.remove(tgSavedEntryTable, i);
		end		
		i = i + 1;
	end
	
	-- deleting category from view table
	i = 1;
	while i <= table.getn(tgSavedViewTable) do
		if (tgSavedViewTable[i].type ~= TYPE_NORMAL and tgSavedViewTable[i].name == category) then
			-- collapse category first
			if (tgSavedViewTable[i].type == TYPE_COLLAPSE) then
				self:guiSetViewOverview(i - FauxScrollFrame_GetOffset(tgViewerOverviewScrollFrame));
			end
			table.remove(tgSavedViewTable, i);
		end		
		i = i + 1;
	end
end

function TravelGuide:guiUpdateEntryOverview()

	local keyword = nil;
	if (currentEntry ~= nil) then
		keyword = currentEntry.zone;
	end
	
	local i = 1;
	while i <= table.getn(tgSavedEntryTable) do
		
		if (tgSavedEntryTable[i].type == TYPE_EXPAND and tgSavedEntryTable[i].name == keyword) then
			-- toggle entry once to expand it
			self:guiSetEntryOverview(i - FauxScrollFrame_GetOffset(tgEditorOverviewScrollFrame));
		elseif (tgSavedEntryTable[i].type == TYPE_COLLAPSE) then
			-- toggle entry twice to refresh it
			self:guiSetEntryOverview(i - FauxScrollFrame_GetOffset(tgEditorOverviewScrollFrame));
			self:guiSetEntryOverview(i - FauxScrollFrame_GetOffset(tgEditorOverviewScrollFrame));
		end
		
		
		if (tgSavedEntryTable[i].type == TYPE_COLLAPSE) then
			-- toggle entry twice to refresh it
			self:guiSetEntryOverview(i - FauxScrollFrame_GetOffset(tgEditorOverviewScrollFrame));
			self:guiSetEntryOverview(i - FauxScrollFrame_GetOffset(tgEditorOverviewScrollFrame));
		end
		i = i + 1;
	end
	
	i = 1;
	while i <= table.getn(tgSavedViewTable) do
		
		if (tgSavedViewTable[i].type == TYPE_EXPAND and tgSavedViewTable[i].name == keyword) then
			-- toggle entry once to expand it
			self:guiSetViewOverview(i - FauxScrollFrame_GetOffset(tgViewerOverviewScrollFrame));
		elseif (tgSavedViewTable[i].type == TYPE_COLLAPSE) then
			-- toggle entry twice to refresh it
			self:guiSetViewOverview(i - FauxScrollFrame_GetOffset(tgViewerOverviewScrollFrame));
			self:guiSetViewOverview(i - FauxScrollFrame_GetOffset(tgViewerOverviewScrollFrame));
		end
		
		
		if (tgSavedViewTable[i].type == TYPE_COLLAPSE) then
			-- toggle entry twice to refresh it
			self:guiSetViewOverview(i - FauxScrollFrame_GetOffset(tgViewerOverviewScrollFrame));
			self:guiSetViewOverview(i - FauxScrollFrame_GetOffset(tgViewerOverviewScrollFrame));
		end
		i = i + 1;
	end
end

function TravelGuide:guiShowEntryOverview()

	-- update scroll frame
	FauxScrollFrame_Update(tgEditorOverviewScrollFrame, table.getn(tgSavedEntryTable), MAX_ENTRIES_SHOWN, TRAVEL_GUIDE_ENTRY_HEIGHT, nil, nil, nil, tgEditorEntryHighlightFrame, 293, 316);
	
	-- update overviews
	tgEditorOverviewHighlightFrame:Hide();
	
	local i;
	for i = 1, MAX_ENTRIES_SHOWN do
		local index = i + FauxScrollFrame_GetOffset(tgEditorOverviewScrollFrame);
		
		local title = getglobal("tgEditorOverview"..i);
		local checkMark = getglobal("tgEditorOverview"..i.."Check");
		local normalText = getglobal("tgEditorOverview"..i.."NormalText");
		local highlight = getglobal("tgEditorOverview"..i.."Highlight");
		
		if (index <= table.getn(tgSavedEntryTable)) then
		
			local data = tgSavedEntryTable[index];
			local color;
			
			checkMark:Hide();
			-- icons
			if (data.type == TYPE_EXPAND) then
				title:SetText(data.name);
				title:SetNormalTexture("Interface\\Buttons\\UI-PlusButton-Up");
				highlight:SetTexture("Interface\\Buttons\\UI-PlusButton-Hilight");
				color = QuestDifficultyColor["header"]
			elseif (data.type == TYPE_COLLAPSE) then
				title:SetText(data.name);
				title:SetNormalTexture("Interface\\Buttons\\UI-MinusButton-Up");
				highlight:SetTexture("Interface\\Buttons\\UI-PlusButton-Hilight");
				color = QuestDifficultyColor["header"]
			else
				title:SetText("  "..data.name);
				title:SetNormalTexture("");
				highlight:SetTexture("");
				color = QuestDifficultyColor["difficult"]				
			end
			
			title:SetNormalFontObject(color.font);
			title.r = color.r;
			title.g = color.g;
			title.b = color.b;
			
			if (data.index == tgSavedCurrentEntryIndex) then
				tgEditorOverviewHighlightFrame:SetPoint("TOPLEFT", "tgEditorOverview"..i, "TOPLEFT", 0, 0);
				tgEditorOverviewHighlight:SetVertexColor(title.r, title.g, title.b);
				tgEditorOverviewHighlightFrame:Show();
				title:LockHighlight();
			else
				title:UnlockHighlight();
			end
			
			title:Show();
		else
			title:Hide();
		end
	end
	
	-- disable edit window and entry tag if no guide is selected
	if (tgSavedCurrentEntryIndex == nil) then
		tgEditorEntryFrame:Hide();
	else
		tgEditorEntryFrame:Show();
	end
end

function TravelGuide:guiShowEntryData()
	if (tgSavedCurrentEntryIndex ~= nil) then
		tgEditorEntryTitle:SetText(currentEntry.name);
		tgEditorEntryBrief:SetText(currentEntry.brief);
		tgEditorEntryDetails:SetText(currentEntry.details);
		tgEditorEntryHeaderTitleContent:SetText(currentGuide.name);
		tgEditorEntryHeaderPositionContent:SetText(self:getEditPositionText(currentEntry));
		
		if (tgEditorDelete ~= nil) then
			tgEditorDelete:Enable();
			tgEditorSave:Enable();
			tgEditorEntryResetPosition:Enable();
			
			tgEditorOverviewShiftUp:Show();
			tgEditorOverviewShiftDown:Show();
			tgEditorOverviewZoomIn:Show();
			tgEditorOverviewZoomOut:Show();
			tgEditorEntryResetPosition:Show();
		end
	else
		if (tgEditorDelete ~= nil) then
			tgEditorDelete:Disable();
			tgEditorSave:Disable();
			tgEditorEntryResetPosition:Disable();
			
			tgEditorOverviewShiftUp:Hide();
			tgEditorOverviewShiftDown:Hide();
			tgEditorOverviewZoomIn:Hide();
			tgEditorOverviewZoomOut:Hide();
			tgEditorEntryResetPosition:Hide();
		end
	end
end

function TravelGuide:guiCreateNewEntry()

	-- cannot obtain coordinate
	if (currX == nil or currY == nil) then
		duUtil:message(CREATE_ENTRY_NO_LOCATION_ERROR);
		return;
	end;
	
	-- no guide active
	if (currentGuide == nil or currentGuide.locked) then
		duUtil:message(CREATE_ENTRY_NO_GUIDE_ERROR);
		return;
	end

	-- show viewer
	tgContainer:Show();
	-- show editor
	self:guiSetFrame(tgContainerTab2);
	-- set tab
	self:guiSetEntryTab();
	
	-- create new entry
	local index = self:modifyEntry(nil, nil, nil, nil, currZone, currSubZone, currC, currZ, currX, currY);	
	self:setEntry(index);
	
	-- set to new entry
	self:guiInsertEntryCategory(currentEntry.zone);
	self:guiUpdateEntryOverview();
	self:guiShowEntryOverview();
end

function TravelGuide:guiEntryZoom(change)

	local radius = currentEntry.r + change * ENTRY_MOD_RADIUS;
	
	if (radius < ENTRY_MIN_RADIUS) then
		radius = ENTRY_MIN_RADIUS;
	elseif (radius > ENTRY_MAX_RADIUS) then
		radius = ENTRY_MAX_RADIUS;
	end
	
	self:modifyEntry(tgSavedCurrentEntryIndex, nil, nil, nil, nil, nil, nil, nil, nil, nil, radius);
	self:guiShowEntryData();
end

function TravelGuide:guiEntryBriefLimitLine()

	local lines = duUtil:split(tgEditorEntryBrief:GetText(), "\n");
	
	if (table.getn(lines) > ENTRY_BRIEF_LINE_LIMIT) then
		tgEditorEntryBrief:SetText(string.format("%s\n%s\n%s", lines[1], lines[2], lines[3]));
	end
end

function TravelGuide:guiClearEntryFrameFocus()
	tgEditorEntryTitle:ClearFocus();
	tgEditorEntryBrief:ClearFocus();
	tgEditorEntryDetails:ClearFocus();
end

function TravelGuide:guiResetPosition()

	self:guiClearEntryFrameFocus();
	
	-- cannot obtain coordinate
	if (currX == nil or currY == nil) then
		duUtil:message("Cannot fix location");
		return;
	end;
	
	-- no guide active
	if (currentGuide == nil or currentGuide.locked) then
		duUtil:message("No guide to use");
		return;
	end
	
	-- remove icon
	TravelGuide:guiRemoveIcon(currentEntry);
	
	-- modify position
	self:modifyEntry(tgSavedCurrentEntryIndex, nil, nil, nil, currZone, currSubZone, currC, currZ, currX, currY);
	
	-- replace icon
	TravelGuide:guiAddIcon(currentEntry, true);
	
	local zone = currentEntry.zone;
	self:guiDeleteEntryCategory(zone);
	self:guiInsertEntryCategory(currentEntry.zone);
	
	-- update display
	self:guiShowEntryData();
end

function TravelGuide:guiDeleteEntry()

	self:guiClearEntryFrameFocus();
	
	-- delete guide
	local zone = currentEntry.zone;
	self:deleteEntry(tgSavedCurrentEntryIndex);
	self:guiDeleteEntryCategory(zone);
	self:guiUpdateEntryOverview();
	self:guiShowEntryOverview();
	self:guiShowEntryData();
end

function TravelGuide:guiSaveEntry()

	self:guiClearEntryFrameFocus();
	
	-- save guide
	self:modifyEntry(tgSavedCurrentEntryIndex, tgEditorEntryTitle:GetText(), tgEditorEntryBrief:GetText(), tgEditorEntryDetails:GetText());
	self:guiUpdateEntryOverview();
	self:guiShowEntryOverview();
end

--[[
	guide dropdown
]]

function TravelGuide:guiInitGuideList()

	local options = {};
	local i;
	
	for i = 1, table.getn(tgSavedGuides) do	
		local option = {};
		option.text = tgSavedGuides[i].name;
		option.value = i;
		option.func = function() TravelGuide:guiClickGuideListOption(this) end;
		table.insert(options, option);		
	end
	
	EasyMenu(options, tgViewerGuideSelectDropDown, tgViewerGuideSelect, 0, 0, nil, 5);

end

function TravelGuide:guiShowGuideSelect(btn)
	
	self:guiInitGuideSelect();
	ToggleDropDownMenu(nil, nil, tgViewerGuideSelect);
end

function TravelGuide:guiClickGuideListOption(option)
	self:setGuide(option.value);
end

--[[
	viewer
]]
function TravelGuide:guiSetViewOverview(id)

	local index = id + FauxScrollFrame_GetOffset(tgViewerOverviewScrollFrame);
	local i;
	
	-- expand
	if (tgSavedViewTable[index].type == TYPE_EXPAND) then	
		-- change current icon into a collapse
		tgSavedViewTable[index].type = TYPE_COLLAPSE;
		
		-- expand content
		local pushed = 0;
		
		for i = 1, table.getn(currentGuide.entries) do
			if (currentGuide.entries[i].zone == tgSavedViewTable[index].name) then
				table.insert(tgSavedViewTable, index+pushed+1, {name = currentGuide.entries[i].name, type = TYPE_NORMAL, index = i});
				pushed = pushed + 1;
			end
		end
		
		-- expand failed
		if (pushed == 0) then
			tgSavedViewTable[index].type = TYPE_EXPAND;
		end
		
	elseif (tgSavedViewTable[index].type == TYPE_COLLAPSE) then
		-- change current icon to an expand
		tgSavedViewTable[index].type = TYPE_EXPAND;
		
		-- collapse content
		while (index+1 <= table.getn(tgSavedViewTable) and tgSavedViewTable[index+1].type == TYPE_NORMAL) do
			table.remove(tgSavedViewTable, index+1);
		end
	elseif (tgSavedViewTable[index].type == TYPE_NORMAL) then
		-- changed current guide to index
		self:setEntry(tgSavedViewTable[index].index);
	end
	
	-- update display
	-- self:guiShowViewOverview();
end

function TravelGuide:guiShowViewOverview()

	-- update scroll frame
	FauxScrollFrame_Update(tgViewerOverviewScrollFrame, table.getn(tgSavedViewTable), MAX_ENTRIES_SHOWN, TRAVEL_GUIDE_ENTRY_HEIGHT, nil, nil, nil, tgViewerEntryHighlightFrame, 293, 316);
	
	-- update overviews
	tgViewerOverviewHighlightFrame:Hide();
	
	local i;
	for i = 1, MAX_ENTRIES_SHOWN do
		local index = i + FauxScrollFrame_GetOffset(tgViewerOverviewScrollFrame);
		
		local title = getglobal("tgViewerOverview"..i);
		local checkMark = getglobal("tgViewerOverview"..i.."Check");
		local normalText = getglobal("tgViewerOverview"..i.."NormalText");
		local highlight = getglobal("tgViewerOverview"..i.."Highlight");
		
		checkMark:Hide();
		
		if (index <= table.getn(tgSavedViewTable)) then
		
			local data = tgSavedViewTable[index];
			local color;
			
			-- icons
			if (data.type == TYPE_EXPAND) then
				title:SetText(data.name);
				title:SetNormalTexture("Interface\\Buttons\\UI-PlusButton-Up");
				highlight:SetTexture("Interface\\Buttons\\UI-PlusButton-Hilight");
				color = QuestDifficultyColor["header"]
			elseif (data.type == TYPE_COLLAPSE) then
				title:SetText(data.name);
				title:SetNormalTexture("Interface\\Buttons\\UI-MinusButton-Up");
				highlight:SetTexture("Interface\\Buttons\\UI-PlusButton-Hilight");
				color = QuestDifficultyColor["header"]
			else
				title:SetText("  "..data.name);
				title:SetNormalTexture("");
				highlight:SetTexture("");
				color = QuestDifficultyColor["difficult"]
				
				if (currentGuide.entries[data.index].unlocked) then
					checkMark:Show();
				end
			end
			
			title:SetNormalFontObject(color.font);
			title.r = color.r;
			title.g = color.g;
			title.b = color.b;
			
			if (data.index == tgSavedCurrentEntryIndex) then
				tgViewerOverviewHighlightFrame:SetPoint("TOPLEFT", "tgViewerOverview"..i, "TOPLEFT", 0, 0);
				tgViewerOverviewHighlight:SetVertexColor(title.r, title.g, title.b);
				tgViewerOverviewHighlightFrame:Show();
				title:LockHighlight();
			else
				title:UnlockHighlight();
			end
			
			title:Show();
		else
			title:Hide();
		end
	end
	
	if (currentGuide) then
		tgViewerTrackAll:Enable();
		tgViewerTrackZone:Enable();
	else
		tgViewerTrackAll:Disable();
		tgViewerTrackZone:Disable();
	end
end

function TravelGuide:guiShowViewData()
	if (tgSavedCurrentGuideIndex ~= nil) then
		tgViewerGuideTitleContent:SetText(currentGuide.name);
	else
		tgViewerGuideTitleContent:SetText(NO_GUIDE_TEXT);
	end

	if (tgSavedCurrentEntryIndex ~= nil) then
		tgViewerTitleContent:SetText(currentEntry.name);
		tgViewerPositionContent:SetText(string.format(VIEW_LOCATION_FORMAT, self:getPositionText(currentEntry)));

		if (currentEntry.unlocked and currentEntry.brief ~= "") then
			tgViewerBriefLabel:Show();
			tgViewerBriefContent:Show();
			tgViewerBriefContent:SetText(currentEntry.brief);
		else
			tgViewerBriefLabel:Hide();
			tgViewerBriefContent:Hide();
		end
			
		if (currentEntry.unlocked and currentEntry.details ~= "") then
			tgViewerDetailsLabel:Show();
			tgViewerDetailsContent:Show();
			tgViewerDetailsContent:SetText(currentEntry.details);
		else
			tgViewerDetailsLabel:Hide();
			tgViewerDetailsContent:Hide();
		end
		
		tgViewerDescriptionScrollFrame:Show();
	else
		tgViewerDescriptionScrollFrame:Hide();
	end
end

function TravelGuide:guiViewScroll(self, offset)
	
	FauxScrollFrame_OnVerticalScroll(self, offset, TRAVEL_GUIDE_ENTRY_HEIGHT, TravelGuide.guiShowViewOverview);
end

--[[
	live update
]]

function TravelGuide:guiSetLiveEntry(entry)
	tgLiveUpdateEntryTitle:SetText(entry.name);
	tgLiveUpdateEntryDescription:SetText(entry.brief);
end

function TravelGuide:guiUpdateLiveAlpha(interval)

	if (updateFadeOut) then
		updateAlpha = updateAlpha - interval;
		if (updateAlpha < 0) then
			updateFadeOut = false;
			updateAlpha = 0;
		end
	elseif (updateFadeIn) then
		updateAlpha = updateAlpha + interval * 8;
		if (updateAlpha > 1.0) then
			updateFadeIn = false;
			updateAlpha = 1.0;
		end
	end

	tgLiveUpdate:SetAlpha(updateAlpha);
	
	if (popUpTimer > 0) then
		popUpTimer = popUpTimer - interval;
		if (popUpTimer < 0) then
			popUpFadeOut = true;
		end
	end
	
	if (popUpFadeOut) then
		local alpha = tgLivePopUp:GetAlpha();
		alpha = alpha - interval;
		if (alpha < 0) then
			popUpFadeOut = false;
			alpha = 0;
		end		
		tgLivePopUp:SetAlpha(alpha);
	end
end

function TravelGuide:guiViewDetails()

	-- cannot obtain data
	if (currentView == nil) then
		duUtil:message("TravelGuide Internal Error: Code 071");
		return;
	end;
	
	-- no guide active
	if (currentGuide == nil or currentGuide.locked) then
		duUtil:message("TravelGuide Internal Error: Code 069");
		return;
	end

	-- show viewer
	tgContainer:Show();
	-- show viewer
	self:guiSetFrame(tgContainerTab1);
	
	-- set to new entry
	local i;
	for i = 1, table.getn(currentGuide.entries) do
		if (currentGuide.entries[i] == currentView) then
			self:setEntry(i);
		end
	end
	
	-- view entry
	self:guiShowViewOverview();
end

function TravelGuide:guiShowPopUp(text)
	if (text ~= nil) then
		tgLivePopUpContent:SetText(text);
	end
	
	-- reset duration, show popup
	popUpFadeOut = false;
	popUpTimer = POPUP_SHOW_TIME;
	tgLivePopUp:SetAlpha(1.0);
end

--[[
	mapping
]]

function TravelGuide:guiTrackZone()
	for i = 1, table.getn(currentGuide.entries) do
		if (currentGuide.entries[i].z == currZ) then
			self:guiAddIcon(currentGuide.entries[i], false);
		end
	end
end

function TravelGuide:guiTrackAll()
	local i;
	for i = 1, table.getn(currentGuide.entries) do
		self:guiAddIcon(currentGuide.entries[i], false);
	end
end

function TravelGuide:guiTrackClear()
	while table.getn(currentIcons) > 0 do
		self:guiRemoveIcon(currentIcons[1].minimap.entry);
	end
end

function TravelGuide:guiAddIcon(entry, temp)

	-- find index
	local s, i = nil, 0;
	
	for i = 1, table.getn(currentGuide.entries) do
		if (currentGuide.entries[i] == entry) then
			s = i;
		end
	end
	
	-- entry not found
	if (not s) then
		return;
	end
	
	-- already tracked
	for i = 1, table.getn(currentIcons) do
		if (currentIcons[i].minimap.entry == entry) then
			if (not temp) then
				currentIcons[i].temp = temp;
			end
			return;
		end
	end
	
	-- create minimap icon and world map icon
	local icon;

	if (table.getn(recycleIcons) <= 0) then
		icon = {};
		iconCount = iconCount + 1;
		icon.minimap = CreateFrame("Button", string.format(MINIMAP_ICON_NAME_FORMAT, iconCount), nil, "tgMinimapIconTemplate");
		iconCount = iconCount + 1;
		icon.worldMap = CreateFrame("Button", string.format(WORLD_MAP_ICON_NAME_FORMAT, iconCount), WorldMapDetailFrame, "tgWorldMapIconTemplate");
		icon.worldMap:RegisterEvent("WORLD_MAP_UPDATE");
	else
		icon = recycleIcons[1];
		icon.minimap:Show();
		icon.worldMap:Show();
		table.remove(recycleIcons, 1);
	end
	
	icon.temp = temp;
	icon.minimap.entry = entry;
	icon.worldMap.entry = entry;

	-- clean up appearance
	if (entry ~= currentEntry) then
		getglobal(icon.minimap:GetName().."Normal"):Show();
		getglobal(icon.minimap:GetName().."Selected"):Hide();
		getglobal(icon.worldMap:GetName().."Normal"):Show();
		getglobal(icon.worldMap:GetName().."Selected"):Hide();
	else
		getglobal(icon.minimap:GetName().."Normal"):Hide();
		getglobal(icon.minimap:GetName().."Selected"):Show();
		getglobal(icon.worldMap:GetName().."Normal"):Hide();
		getglobal(icon.worldMap:GetName().."Selected"):Show();
	end
	
	-- place icons
	Astrolabe:PlaceIconOnMinimap(icon.minimap, entry.c, entry.z, entry.x, entry.y);
	Astrolabe:PlaceIconOnWorldMap(WorldMapDetailFrame, icon.worldMap, entry.c, entry.z, entry.x, entry.y);
	
	-- enter icons into management system
	table.insert(currentIcons, icon);

end

function TravelGuide:guiFindIcon(entry)
	-- find index
	local i;
	for i = 1, table.getn(currentIcons) do
		if (currentIcons[i].minimap.entry == entry) then
			return currentIcons[i];
		end
	end
	
	return nil;
end

function TravelGuide:guiRemoveIcon(entry)

	-- find index
	local icon = self:guiFindIcon(entry);
	
	if (not icon) then
		duUtil:message("wtf");
		return;
	end
	
	-- remove management
	Astrolabe:RemoveIconFromMinimap(icon.minimap);

	-- recycle buttons
	icon.minimap:Hide();
	icon.worldMap:Hide();
	icon.minimap.entry = nil;
	icon.worldMap.entry = nil;
	table.insert(recycleIcons, icon);
	
	-- remove from management system
	local i;
	for i = 1, table.getn(currentIcons) do
		if (currentIcons[i] == icon) then
			table.remove(currentIcons, i);
			return;
		end
	end
end

function TravelGuide:guiShowIconTooltip(icon)

	worldMapTooltip:SetParent(icon:GetParent());
	worldMapTooltip:SetOwner(icon, "ANCHOR_TOPLEFT");
	worldMapTooltip:SetText(icon.entry.name);
end

function TravelGuide:guiHideIconTooltip()

	worldMapTooltip:Hide();
end

function TravelGuide:guiUpdateMinimapIcon(icon, interval)

	local dist, x, y = Astrolabe:GetDistanceToIcon(icon);
	
	if (not dist) or (not currX) or (not currY) then
		icon:SetAlpha(0);
		return;
	end
	
	local edge = Astrolabe:IsIconOnEdge(icon);
	if edge then
		icon:SetAlpha(0);
	else
		icon:SetAlpha(0.75);
		--icon:SetWidth(ICON_CONVERSION_FACTOR * icon.entry.r);
		--icon:SetHeight(ICON_CONVERSION_FACTOR * icon.entry.r);
	end
end
		
function TravelGuide:guiUpdateWorldMapIcon(icon)

	-- recycled icon
	if not (icon.entry) then
		return;
	end;
	
	if event == "WORLD_MAP_UPDATE" then
		local entry = icon.entry;
		local x, y = Astrolabe:PlaceIconOnWorldMap(WorldMapDetailFrame, icon, icon.entry.c, icon.entry.z, icon.entry.x, icon.entry.y);
		if (x and y and (0 < x and x <= 1) and (0 < y and y <= 1)) then
			icon:Show();
			icon:SetAlpha(0.75);
			--icon:SetWidth(ICON_CONVERSION_FACTOR * icon.entry.r);
			--icon:SetHeight(ICON_CONVERSION_FACTOR * icon.entry.r);
		else
			icon:Hide()
		end
	end
end

--[[
	minimap icon
]]

function TravelGuide:guiUpdateMinimapEdgeIcon()
	if (edgeDragging) then
		self:dragEdgeIcon();
	end
end

function TravelGuide:dragEdgeIcon()
	local xpos,ypos = GetCursorPosition()
	local xmin,ymin = Minimap:GetLeft(), Minimap:GetBottom()

	xpos = xmin-xpos/Minimap:GetEffectiveScale()+70
	ypos = ypos/Minimap:GetEffectiveScale()-ymin-70

	tgSavedPrefs.edgePosition = math.deg(math.atan2(ypos,xpos))
	self:moveEdgeIcon();
end

function TravelGuide:moveEdgeIcon()
	local xpos,ypos
	local angle = tgSavedPrefs.edgePosition or -100
	
	xpos = 80*cos(angle)
	ypos = 80*sin(angle)
	
	tgMinimapEdgeButton:SetPoint("TOPLEFT","Minimap","TOPLEFT",52-xpos,ypos-52)
	tgMinimapEdgeButton:Show()
end

function TravelGuide:guiStartDragMinimapEdgeIcon()
	edgeDragging = true;
end

function TravelGuide:guiStopDragMinimapEdgeIcon()
	edgeDragging = false;
end

function TravelGuide:guiToggleContainer()
	if (tgContainer:IsShown() or connectState) then
		tgContainer:Hide();
		tgViewer:Hide();
		tgEditor:Hide();
	else
		tgContainer:Show();
		TravelGuide:guiSetFrame(currentMode);
	end
end

--[[
	sharing
]]

function TravelGuide:connectInputLocked()
	return connectInputLock;
end

function TravelGuide:connectInputLock()
	connectInputLock = true;
	tgShareRecipient:ClearFocus();
end

function TravelGuide:connectInputUnlock()
	connectInputLock = false;
end

function TravelGuide:guiShowShare()

	if (connectTarget) then
		tgShareRecipient:SetText(connectTarget);
	else
		tgShareRecipient:SetText("");
	end
	
	if (connectState == CONN_SEND) then
		tgShareRecipientLabel:SetText(MSG_RECIPIENT_LABEL);
		self:connectInputUnlock();
		
		if (UnitIsPlayer("target")) then
			tgShareRecipient:SetText(UnitName("target"));
		end
		
		tgShareStart:Enable();
	elseif (connectState == CONN_RECIEVE) then
		tgShareRecipientLabel:SetText(MSG_SENDER_LABEL);
		tgShareStart:Disable();
		self:connectInputLock();		
	end
	
	tgShareProgressBar:SetWidth(1);
	tgShareProgress:SetText("");
	tgShareCancel:SetText(SHARE_CANCEL_BUTTON_LABEL);
end

function TravelGuide:guiHideShare()

	if (connectTarget) then
		-- terminate connection
		self:sendMessage(MSG_END);
		connectTarget = nil;
		connectState = nil;
		-- unlock recipient
		self:connectInputUnlock();
	end
end

function TravelGuide:guiStartShare()

	-- select user
	connectTarget = tgShareRecipient:GetText();
	
	if (connectTarget == "") then
		connectTarget = nil;
		tgShareProgress:SetText(NO_RECIPIENT_LABEL);
		return;
	end
	
	-- lock recipient
	self:connectInputLock();
	-- request permission
	self:sendMessage(MSG_REQUEST..currentGuide.name);
	-- change status message
	tgShareProgressBar:SetWidth(1);
	tgShareProgress:SetText(REQUEST_PERMISSION_LABEL);
	tgShareStart:Disable();
end

function TravelGuide:guiCancelShare()

	-- restore interface
	tgShare:Hide();
	if (connectState == CONN_SEND) then
		tgContainer:Show();
		TravelGuide:guiSetFrame(currentMode);
	end
end

function TravelGuide:guiAcceptGuideShare()
	
	-- hides display
	tgContainer:Hide();
	tgViewer:Hide();
	tgEditor:Hide();
	
	-- opens sharing interface
	tgShare:Show();
	
	-- send accept message
	self:sendMessage(MSG_ACCEPT);
end

function TravelGuide:guiRejectGuideShare()
	-- send reject message
	self:sendMessage(MSG_REJECT);
	connectState = nil;
	connectTarget = nil;
end

function TravelGuide:sendMessage(msg)
	SendAddonMessage(MSG_ADDON_NAME, msg, "WHISPER", connectTarget);
end

function TravelGuide:interpretMessage(msg, sender)

	local header = string.sub(msg, 1, 3);
	local content = string.sub(msg, 4);
	
	if (header == MSG_REQUEST) then
		if (connectState) then
			-- deny incoming request if connection exists
			SendAddonMessage(MSG_ADDON_NAME, MSG_REJECT, "WHISPER", sender);
		else
			-- ask user for permission
			connectState = CONN_RECIEVE;
			connectTarget = sender;
			StaticPopupDialogs["TRAVEL_GUIDE_RECIEVE_GUIDE_CONFIRM"].text = string.format(TRAVEL_GUIDE_RECIEVE_GUIDE_DIALOG_FORMAT, sender, content);
			StaticPopup_Show("TRAVEL_GUIDE_RECIEVE_GUIDE_CONFIRM");
		end
	
	-- the rest of these messages are one to one communications
	elseif (connectTarget and string.lower(sender) == string.lower(connectTarget)) then
		if (header == MSG_REJECT) then
			-- update message
			tgShareProgress:SetText(REQUEST_REJECTED_LABEL);
			tgShareStart:Enable();
			-- enable interface
			self:connectInputUnlock();
			
		elseif (header == MSG_ACCEPT) then
			-- update message
			tgShareProgress:SetText(REQUEST_ACCEPTED_LABEL);
			-- prepare message
			self:compressGuide();
			
		elseif (header == MSG_END) then
			-- terminate process
			if (connectState == CONN_SEND) then
				-- enable interface
				self:connectInputUnlock();
				tgShareStart:Enable();
				tgShareProgress:SetText(CONNECTION_FAILED_LABEL);
			elseif (connectState == CONN_RECIEVE) then
				-- enable interface
				tgShareProgress:SetText(CONNECTION_FAILED_LABEL);
			end
			
		elseif (header == MSG_START) then
			local data = duUtil:split(content, MSG_DIVIDER);
			-- length
			recieveGoal = data[1] + 0;
			recieveProgress = 1;
			-- stuff the array into the queue
			recieveQueue = {};
			table.insert(recieveQueue, data);
			self:updateRecieveProgress();
			
		elseif (header == MSG_INCOMING) then
			local data = duUtil:split(content, MSG_DIVIDER);
			local i;
			-- apply previous splits, if applicable
			if (recievePartial) then
				for i = 1, table.getn(data) do
					table.insert(recievePartial, data[i]);
				end
				data = recievePartial;
			end		
			-- wipe out partials
			recievePartial = nil;
			table.insert(recieveQueue, data);
			recieveProgress = recieveProgress + 1;
			self:updateRecieveProgress();
			
		elseif (header == MSG_INCOMING_SPLIT) then
		
			--duUtil:message(msg);
			local data = duUtil:split(content, MSG_DIVIDER);
			if (not recievePartial) then
				recievePartial = data;
			else
				-- expand array
				local i;
				for i = 1, table.getn(data) do
					table.insert(recievePartial, data[i]);
				end
			end		
			recieveProgress = recieveProgress + 1;
			self:updateRecieveProgress();
		end
	end
end

function TravelGuide:updateRecieveProgress()
	-- update display
	local percentage = recieveProgress/recieveGoal;
	tgShareProgressBar:SetWidth(percentage * SHARE_BAR_WIDTH);
	tgShareProgress:SetText(string.format(RECIEVE_PROGRESS_FORMAT, percentage * 100));
	
	if (recieveProgress == recieveGoal) then
		tgShareProgress:SetText(RECIEVE_COMPLETE_LABEL);
		tgShareCancel:SetText(SHARE_DONE_BUTTON_LABEL);
		self:uncompressGuide();
	end
		
end

function TravelGuide:compressGuide()
	
	sendQueue = {};
	
	-- reserve slot 1
	table.insert(sendQueue, "");
	
	local SEND_HEADER_FORMAT = "%s%s%s%s%s%s%s%s%s%i%s%i%s%.8f%s%.8f%s%i";
	local SEND_DETAILS_FORMAT = "%s%s";
	
	-- parse all other entries
	local i;
	for i = 1, table.getn(currentGuide.entries) do
		local entry = currentGuide.entries[i];
		-- send basic information
		table.insert(sendQueue, string.format(SEND_HEADER_FORMAT, MSG_INCOMING_SPLIT, entry.name, MSG_DIVIDER, entry.brief, MSG_DIVIDER, entry.zone, MSG_DIVIDER, entry.subzone, MSG_DIVIDER, entry.c, MSG_DIVIDER, entry.z, MSG_DIVIDER, entry.x, MSG_DIVIDER, entry.y, MSG_DIVIDER, entry.r));
        if (string.len(entry.details) > 675) then
            -- send in four parts
			table.insert(sendQueue, string.format(SEND_DETAILS_FORMAT, MSG_INCOMING_SPLIT, string.sub(entry.details, 1, 225)));
			table.insert(sendQueue, string.format(SEND_DETAILS_FORMAT, MSG_INCOMING_SPLIT, string.sub(entry.details, 226, 450)));
			table.insert(sendQueue, string.format(SEND_DETAILS_FORMAT, MSG_INCOMING_SPLIT, string.sub(entry.details, 451, 675)));
			table.insert(sendQueue, string.format(SEND_DETAILS_FORMAT, MSG_INCOMING, string.sub(entry.details, 676)));
        elseif (string.len(entry.details) > 450) then
            -- send in three parts
			table.insert(sendQueue, string.format(SEND_DETAILS_FORMAT, MSG_INCOMING_SPLIT, string.sub(entry.details, 1, 225)));
			table.insert(sendQueue, string.format(SEND_DETAILS_FORMAT, MSG_INCOMING_SPLIT, string.sub(entry.details, 226, 450)));
			table.insert(sendQueue, string.format(SEND_DETAILS_FORMAT, MSG_INCOMING, string.sub(entry.details, 451)));
		elseif (string.len(entry.details) > 225) then
			-- send in two parts
			table.insert(sendQueue, string.format(SEND_DETAILS_FORMAT, MSG_INCOMING_SPLIT, string.sub(entry.details, 1, 225)));
			table.insert(sendQueue, string.format(SEND_DETAILS_FORMAT, MSG_INCOMING, string.sub(entry.details, 226)));
		else
			table.insert(sendQueue, string.format(SEND_DETAILS_FORMAT, MSG_INCOMING, entry.details));
		end
	end
		
	-- change slot 1: guide info, send queue length
	local boolEntry = "0";
	if (currentGuide.locked) then
		boolEntry = "1";
	end
	
	local SEND_START_FORMAT = "%s%i%s%s%s%s%s%s%s%s";
	sendQueue[1] = string.format(SEND_START_FORMAT, MSG_START, table.getn(sendQueue), MSG_DIVIDER, currentGuide.name, MSG_DIVIDER, currentGuide.details, MSG_DIVIDER, boolEntry, MSG_DIVIDER, currentGuide.password);

	-- begin sending
	sendProgress = 1;
	sendInterval = MSG_SEND_DELAY;
end

function TravelGuide:uncompressGuide()

	-- create guide
	local boolEntry = false;
	if (recieveQueue[1][4] == "1") then
		boolEntry = true;
	end
	
	local g = self:modifyGuide(nil, recieveQueue[1][2], recieveQueue[1][3], boolEntry, recieveQueue[1][5]);
	
	-- save guide
	local savedCurrentGuide = currentGuide;
	currentGuide = tgSavedGuides[g];
	
	-- create entries
	local i;
	for i = 2, table.getn(recieveQueue) do
		local entry = recieveQueue[i];
		if (table.getn(entry) == 13) then
			self:modifyEntry(nil, entry[1], entry[2], string.format("%s%s%s%s", entry[10], entry[11], entry[12], entry[13]), entry[3], entry[4], entry[5]+0, entry[6]+0, entry[7]+0, entry[8]+0, entry[9]+0);
		elseif (table.getn(entry) == 12) then
			self:modifyEntry(nil, entry[1], entry[2], string.format("%s%s%s", entry[10], entry[11], entry[12]), entry[3], entry[4], entry[5]+0, entry[6]+0, entry[7]+0, entry[8]+0, entry[9]+0);
		elseif (table.getn(entry) == 11) then
			self:modifyEntry(nil, entry[1], entry[2], string.format("%s%s", entry[10], entry[11]), entry[3], entry[4], entry[5]+0, entry[6]+0, entry[7]+0, entry[8]+0, entry[9]+0);
		else
			self:modifyEntry(nil, entry[1], entry[2], entry[10], entry[3], entry[4], entry[5]+0, entry[6]+0, entry[7]+0, entry[8]+0, entry[9]+0);
		end		
	end

	-- restore guide
	currentGuide = savedCurrentGuide;
	self:guiUpdateGuideOverview();
end