duTrans = {};

-- constants
duTrans.DELIM = "::";

duTrans.MSG_HEADER = "DU_TRANS";
duTrans.MSG_START = duTrans.MSG_HEADER..duTrans.DELIM.."ST";
duTrans.MSG_DATA = duTrans.MSG_HEADER..duTrans.DELIM.."DA";
duTrans.SEND_INTERVAL = 0.25;

-- variables
duTrans.outgoing = {};
duTrans.sending = false;
duTrans.sendAddon = "";
duTrans.sendTarget = "";
duTrans.sendPos = 0;
duTrans.sendLen = 0;
duTrans.sendDelay = 0;

duTrans.incoming = {};
duTrans.recieving = false;
duTrans.getParams = {};
duTrans.getTarget = "";
duTrans.getPos = 0;
duTrans.getLen = 0;

duTrans.listener = nil;

--
-- points callback function
--
function duTrans:setListener(obj)
	self.listener = obj;
end

--
-- begins transmitting message
--
function duTrans:send(addon, target, block, params)

	-- already sending
	if ( self.sending ) then
		return false;
	end
	
	-- process the outgoing block
	self.sendAddon = addon;
	self.sendTarget = target;
	self.sendPos = 1;
	self.sendLen = table.getn(block);

	self.outgoing = {};
	
	local i, name, code;
	
	-- insert start
	code = self.MSG_START..self.DELIM..self.sendLen;
	for name in pairs(params) do
		code = code..self.DELIM..params[name];
	end
	
	table.insert(self.outgoing, code);
	
	for i = 1, self.sendLen do
	
		-- start data block
		code = self.MSG_DATA;

		-- add data
		for name in pairs(params) do
			code = code..self.DELIM..block[i][params[name]];
		end
		
		-- add to outgoing
		table.insert(self.outgoing, code);
	
	end
	
	-- signal send
	self.sendDelay = 0;
	self.sending = true;

end
	
--
-- update transmissions
--
function duTrans:update(interval)
	
	if (self.sending) then
	
		self.sendDelay = self.sendDelay - interval;
		-- duUtil:message("delay : "..self.sendDelay);
		
		if (self.sendDelay <= 0) then
			-- send message
			SendAddonMessage(self.sendAddon, self.outgoing[self.sendPos], "WHISPER", self.sendTarget);
			
			-- update index
			self.sendPos = self.sendPos + 1;
			if (self.sendPos > self.sendLen + 1) then
				-- end transmission
				self.sending = false;
				-- end transmission event
				if (self.listener and self.listener.evtTransSendComplete) then
					self.listener.evtTransSendComplete();
				end
			else
				-- reset timer
				self.sendDelay = self.SEND_INTERVAL;
			end
		end
	end
	
end

--
-- detects and translates incoming messsage
--
function duTrans:decode(text, target)

	if (self.recieving) then
	
		-- expects data
		if (target == self.getTarget and strfind(text, self.MSG_DATA, 1) == 1) then
			
			-- expands data into array
			local data = duUtil:split(text, self.DELIM);
			
			-- recreate data pairs
			local i, block = 1, {};
			for i = 1, table.getn(self.getParams) do
				block[self.getParams[i]] = data[i + 2];
			end
			
			-- insert data into incoming
			table.insert(self.incoming, block);
			
			-- increase index
			self.getPos = self.getPos + 1;
			if (self.getPos > self.getLen) then
				-- end of transmission
				self.recieving = false;
				if (self.listener and self.listener.evtTransGetComplete) then
					self.listener.evtTransGetComplete();
				end
			end
			
			return true;
		end
	
	else
	
		-- listens for incoming data
		if (strfind(text, self.MSG_START, 1) == 1) then
		
			-- expands data into array
			local header = duUtil:split(text, self.DELIM);
			
			-- extract header data
			self.getTarget = target;
			self.getPos = 1;
			self.getLen = header[3] + 0;
			self.getParams = {};
			
			local i;
			for i = 4, table.getn(header) do
				table.insert(self.getParams, header[i]);
			end
			
			-- begin transmission
			self.recieving = true;
			
			-- if data is blank, we're already done!
			if (self.getLen == 0) then
				-- end of transmission
				self.recieving = false;
				if (self.listener and self.listener.evtTransGetComplete) then
					self.listener.evtTransGetComplete();
				end
			end
			
			return true;
		end
		
	end
	
	return false;
end